class CustomHED:
    """
    Create class with following fixed function names and the number of arguents to train your model from external source
    """

    try:
        import torch
        from torchvision import models
        from arcgis.learn.models import _hed_utils as hed
    except:
        pass

    def get_model(self, data, backbone=None, **kwargs):
        """
        In this fuction you have to define your model with following two arguments!

        """
        pretrained_backbone = kwargs.get("pretrained_backbone", True)

        if backbone is None:
            self._backbone = self.models.vgg19
        elif type(backbone) is str:
            if hasattr(self.models, backbone):
                self._backbone = getattr(self.models, backbone)
            elif hasattr(self.models.detection, backbone):
                self._backbone = getattr(self.models.detection, backbone)
            elif "timm:" in backbone:
                import timm

                bckbn = backbone.split(":")[1]
                if hasattr(timm.models, bckbn):
                    self._backbone = getattr(timm.models, bckbn)
        else:
            self._backbone = backbone

        model = self.hed._HEDModel(
            self._backbone, data.chip_size, pretrained=pretrained_backbone
        )

        return model

    def on_batch_begin(self, learn, model_input_batch, model_target_batch, **kwargs):
        return model_input_batch, model_target_batch

    def transform_input(self, xb):
        return xb

    def transform_input_multispectral(self, xb):
        return xb

    def loss(self, model_output, *model_target):
        final_loss = self.hed.hed_loss(model_output, *model_target)

        return final_loss

    def post_process(self, pred, thres=0.5, thinning=True, prob_raster=False):
        """
        In this function you have to return list with appended output for each image in the batch with shape [C=1,H,W]!

        """

        from skimage.morphology import skeletonize, binary_dilation
        import numpy as np

        post_processed_pred = []
        if isinstance(pred, list):
            pred = pred[-1]
        if prob_raster:
            return pred
        elif thinning:
            for p in pred:
                p = self.torch.unsqueeze(
                    self.torch.tensor(
                        skeletonize(
                            binary_dilation(
                                np.squeeze((p >= thres).byte().cpu().numpy())
                            )
                        )
                    ),
                    dim=0,
                )
                post_processed_pred.append(p)
            return post_processed_pred
        else:
            return (pred >= thres).byte()
